/*
 jGuard is a security framework based on top of jaas (java authentication and authorization security).
 it is written for web applications, to resolve simply, access control problems.
 version $Name$
 http://sourceforge.net/projects/jguard/

 Copyright (C) 2004  Charles GAY

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 jGuard project home page:
 http://sourceforge.net/projects/jguard/

 */
package net.sf.jguard.ext.authentication;

import java.io.Serializable;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.security.auth.login.LoginException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author <a href="mailto:tandilero@users.sourceforge.net">Maximiliano Batelli</a>
 * 
 */
public class JGuardLoginException extends LoginException implements Serializable {

	private static final Log logger = LogFactory.getLog(JGuardLoginException.class);
	/**
	 * serial version number.
	 */
	private static final long serialVersionUID = 1L;
	
	private String errorKey = "";

	/**
	 * Constructor for JguardLoginException with error key.
	 * 
	 * @param message The error key
	 */
	public JGuardLoginException(String errorKey) {
		this.errorKey = errorKey;
	}

	public String getMessage() {
		return getLocalizedMessage(Locale.getDefault());
	}
	
	public String getLocalizedMessage(Locale currentLocale) {

		ResourceBundle messages = null;
		String message = null;
		try {
			messages = ResourceBundle.getBundle("JGuard", currentLocale);
		} catch (MissingResourceException e) {
			// there is not specific bundle to get messages, take the failsafe one and inform to user
			logger.warn("There is not specific bundle to get messages: create a JGuard.properties and copy it to your application base classpath");
			messages = ResourceBundle.getBundle("JGuard-failsafe", currentLocale);
		}
		
		try {
			message = messages.getString(errorKey);
		} catch (MissingResourceException e) {
			message = "Login error!!! but missing specific error key in bundle: " + errorKey;
		}
		return message;
	}
}
