/*
 jGuard is a security framework based on top of jaas (java authentication and authorization security).
 it is written for web applications, to resolve simply, access control problems.
 version $Name$
 http://sourceforge.net/projects/jguard/

 Copyright (C) 2004  Charles GAY

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 jGuard project home page:
 http://sourceforge.net/projects/jguard/

 */
package net.sf.jguard.ext.authentication.loginmodules;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.util.JNDIUtils;

/**
 * JNDI - related LoginModule.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @see LoginModule
 */
public class JNDILoginModule extends UserLoginModule implements LoginModule {

	private static final Logger logger = Logger.getLogger(JNDILoginModule.class.getName());
	private DirContext initDirCtx = null;
	private DirContext userCtxt = null;
	private int searchScope = -1;
	private int timeLimit = -1;
	private long countLimit = -1;
	private boolean derefLinkFlag = false;
	private String[] returningAttributes;
	private boolean returningObjFlag = false;
	private String searchContextName = null;
	private String searchFilterExpression = "";
	private String searchFiterArgs = null;
	private boolean pathToUserKnown = true;
	private String userPath = null;
	private String rolePath = null;
	private Set credentials = null;
	private int ppalAppNameLevel = 2;
	private String ppalNameAttribute = "cn";
	private String organizationlaRoleAttribute = "organizationalRole";
	private String userPasswordAttribute = "userPassword";

	/**
	 *
	 * @param subj
	 * @param cbkHandler
	 * @param sState
	 * @param opts
	 */
	public void initialize(Subject subj, CallbackHandler cbkHandler, Map sState, Map opts) {
		super.initialize(subj, cbkHandler, sState, opts);
		if ((String) opts.get(SecurityConstants.DIRECTORY_PATH_TO_USER) != null) {
			pathToUserKnown = Boolean.valueOf((String) opts.get(SecurityConstants.DIRECTORY_PATH_TO_USER)).booleanValue();
		}
		if ((String) opts.get(SecurityConstants.DIRECTORY_SEARCH_SCOPE) != null) {
			searchScope = Integer.parseInt((String) opts.get(SecurityConstants.DIRECTORY_SEARCH_SCOPE));
		}
		if ((String) opts.get(SecurityConstants.DIRECTORY_TIME_LIMIT) != null) {
			timeLimit = Integer.parseInt((String) opts.get(SecurityConstants.DIRECTORY_TIME_LIMIT));
		}
		if ((String) opts.get(SecurityConstants.DIRECTORY_COUNT_LIMIT) != null) {
			countLimit = Long.parseLong((String) opts.get(SecurityConstants.DIRECTORY_COUNT_LIMIT));
		}
		if ((String) opts.get(SecurityConstants.DIRECTORY_DEREF_LINK_FLAG) != null) {
			derefLinkFlag = Boolean.valueOf((String) opts.get(SecurityConstants.DIRECTORY_DEREF_LINK_FLAG)).booleanValue();
		}
		if ((String) opts.get(SecurityConstants.DIRECTORY_RETURNING_ATTRIBUTES) != null) {
			returningAttributes = ((String) opts.get(SecurityConstants.DIRECTORY_RETURNING_ATTRIBUTES)).split(",");
		}

		if ((String) opts.get(SecurityConstants.DIRECTORY_RETURNING_OBJ_FLAG) != null) {
			returningObjFlag = Boolean.valueOf((String) opts.get(SecurityConstants.DIRECTORY_RETURNING_OBJ_FLAG)).booleanValue();
		}

		if ((String) opts.get(SecurityConstants.DIRECTORY_SEARCH_CONTEXT_NAME) != null) {
			searchContextName = (String) opts.get(SecurityConstants.DIRECTORY_SEARCH_CONTEXT_NAME);
		}

		if ((String) opts.get(SecurityConstants.DIRECTORY_SEARCH_FILTER_EXPRESSION) != null) {
			searchFilterExpression = (String) opts.get(SecurityConstants.DIRECTORY_SEARCH_FILTER_EXPRESSION);
		}

		// parameters for 'simple' search
		if ((String) opts.get("userPath") != null) {
			userPath = (String) opts.get("userPath");
		}
		if ((String) opts.get("rolePath") != null) {
			rolePath = (String) opts.get("rolePath");
		}

		if ((Integer) opts.get("ppalAppNameAttribute") != null) {
			ppalAppNameLevel = ((Integer) opts.get("ppalAppNameAttribute")).intValue();
		}

		if ((String) opts.get("ppalNameAttribute") != null) {
			ppalNameAttribute = (String) opts.get("ppalNameAttribute");
		}

		if ((String) opts.get("organizationlaRoleAttribute") != null) {
			organizationlaRoleAttribute = (String) opts.get("organizationlaRoleAttribute");
		}

		if ((String) opts.get("userPasswordAttribute") != null) {
			userPasswordAttribute = (String) opts.get("userPasswordAttribute");
		}

		Hashtable contextParams = new Hashtable();
		Set entries = opts.entrySet();
		Iterator itEntries = entries.iterator();

		// convert opts into an HashTable
		while (itEntries.hasNext()) {
			Map.Entry entry = (Map.Entry) itEntries.next();
			if (entry.getValue() != null) {
				contextParams.put(entry.getKey(), entry.getValue());
			}
		}
		try {
			initDirCtx = new InitialDirContext(contextParams);
		} catch (NamingException e) {
			logger.severe(e.getMessage());
		}
	}

	/**
	 * @return true if success, false if ignored.
	 * @throws LoginException
	 *             when it fails
	 */
	public boolean login() throws LoginException {
		String login = super.getLogin();
		String password = super.getPassword();
		NamingEnumeration results = null;
		Object[] args = { login };
		userPath = MessageFormat.format(userPath, args);
		userPath = JNDIUtils.escapeDn(userPath);

		if (userPath != null && !equals("")) {

			try {
				userCtxt = (DirContext) initDirCtx.lookup(userPath);
				Attributes attributes = userCtxt.getAttributes("");
				Attribute passwordAttribute = attributes.get(userPasswordAttribute);
				String passwordFound = new String((byte[]) passwordAttribute.get());
				if (passwordFound != null && passwordFound.equals(password)) {
					results = attributes.getAll();
					credentials = JNDIUtils.grabCredentialsFromJNDIAttributes(results);
				} else {
					throw new LoginException("passsword does not match");
				}

			} catch (NamingException e) {
				// authentication failed
				throw new LoginException(e.getMessage());
			} finally {
				try {
					results.close();
					userCtxt.close();
				} catch (NamingException e) {
					throw new LoginException(e.getMessage());
				}
			}
		} else {
			results = globalSearch();
		}
		// authentication succeed
		return true;
	}

	/**
	 * use this search when the path to the object is not known. this search is <b>LESS</a> performant.
	 *
	 * @return NaminEnumeration
	 */
	private NamingEnumeration globalSearch() {
		NamingEnumeration results = null;
		SearchControls scontrols = new SearchControls();
		if (searchScope != -1) {
			scontrols.setSearchScope(searchScope);
		}
		if (countLimit != -1) {
			scontrols.setCountLimit(countLimit);
		}
		scontrols.setDerefLinkFlag(derefLinkFlag);
		if (returningAttributes.length > 0) {
			scontrols.setReturningAttributes(returningAttributes);
		}
		scontrols.setReturningObjFlag(returningObjFlag);
		if (timeLimit != -1) {
			scontrols.setTimeLimit(timeLimit);
		}
		try {
			if (searchFiterArgs != null) {
				String[] filterArgs = searchFiterArgs.split("#");
				results = initDirCtx.search(searchContextName, searchFilterExpression, filterArgs, scontrols);
			} else {
				results = initDirCtx.search(searchContextName, searchFilterExpression, scontrols);
			}
		} catch (NamingException e) {
			logger.severe(" bad JNDI name " + searchContextName + " " + searchFilterExpression + " " + scontrols);
		}
		return results;
	}

	/**
	 * @return true if success, false if ignored, LoginException when it fails
	 */
	public boolean commit() throws LoginException {

		NamingEnumeration results = null;
		try {
			// ignore attribute name case
			Attributes matchAttrs = new BasicAttributes(true);
			BasicAttribute basicAttribute1 = new BasicAttribute("objectClass", organizationlaRoleAttribute);
			matchAttrs.put(basicAttribute1);
			// Search for objects that have those matching attributes
			StringBuffer searchPath = new StringBuffer(rolePath).append(",").append(userPath);
			logger.finest("search path=" + searchPath);
			results = initDirCtx.search(searchPath.toString(), matchAttrs);
			Set principalsFromJNDI = JNDIUtils.grabPrincipalsFromUser(results, ppalNameAttribute, ppalAppNameLevel);
			logger.finest(principalsFromJNDI.size() + " principals have been found");
			Set principals = subject.getPrincipals();
			principals.addAll(principalsFromJNDI);

		} catch (NamingException e) {
			throw new LoginException(e.getMessage());
		}

		Set privateCredentials = subject.getPrivateCredentials();
		privateCredentials.addAll(credentials);
		try {
			results.close();
			userCtxt.close();
			initDirCtx.close();
		} catch (NamingException e) {
			throw new LoginException(e.getMessage());
		}
		return true;
	}

}
