/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;

import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.login.Configuration;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.configuration.ConfigurationHelper;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.authentication.CryptUtils;
import net.sf.jguard.ext.util.XMLUtils;

import org.dom4j.Document;
import org.dom4j.Element;

/**
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 */
public class AuthenticationHelper {

	private static final Logger logger = Logger.getLogger(AuthenticationHelper.class.getName());



	public static AuthenticationManager initAuthentication(Map authenticationOptions, String applicationName) throws RuntimeException {

		//grab authentication options
		boolean debugOnAuthentication = Boolean.valueOf((String)authenticationOptions.get(CoreConstants.DEBUG)).booleanValue();
		boolean includeOldConfig = Boolean.valueOf((String)authenticationOptions.get(CoreConstants.INCLUDE_OLD_CONFIG)).booleanValue();
		boolean includeConfigFromJavaParam = new Boolean((String)authenticationOptions.get(CoreConstants.INCLUDE_CONFIG_FROM_JAVA_PARAM)).booleanValue();
		ConfigurationHelper.installConfiguration(includeOldConfig, includeConfigFromJavaParam);

		if (logger.isLoggable(Level.FINEST)) {
			logger.log(Level.FINEST,"applicationName="+ applicationName);
			logger.log(Level.FINEST,"authenticationOptions="+ authenticationOptions);
			logger.log(Level.FINEST,"debugOnAuthentication="+ debugOnAuthentication);
		}

		String salt = ((String)authenticationOptions.get(SecurityConstants.SALT));
		String digestAlgorithm = ((String)authenticationOptions.get(SecurityConstants.DIGEST_ALGORITHM));
		if(digestAlgorithm!= null && !"".equals(digestAlgorithm)){
			try {
				CryptUtils.setDigestAlgorithm(digestAlgorithm);
			} catch (NoSuchAlgorithmException e) {
				logger.log(Level.SEVERE, e.getMessage());
				throw new IllegalArgumentException(e);
			}

			if(	salt!=null && !salt.equals("")){
		        boolean setSaltSuccess = CryptUtils.setSalt(salt.toCharArray());
		        if(!setSaltSuccess){
		        	logger.log(Level.WARNING, " salt for message digest has not been set ");
		        }
			}
		}

		//register the authentication part
		ConfigurationHelper.addConfigurationEntryForWebapp(applicationName, authenticationOptions, debugOnAuthentication);
		logger.log(Level.FINEST,"Configuration class = " + Configuration.getConfiguration().getClass().getName());
		Map authManagerOptions = (Map)authenticationOptions.get(SecurityConstants.AUTHENTICATION_MANAGER_OPTIONS);
		authManagerOptions.put(CoreConstants.APPLICATION_NAME, applicationName);

		try {
			AuthenticationManagerFactory.createAuthenticationManager((String)authenticationOptions.get(SecurityConstants.AUTHENTICATION_MANAGER), authManagerOptions);
		} catch (AuthenticationException e) {
			logger.log(Level.SEVERE,"authenticationManager initialization failed");
		}

		return AuthenticationManagerFactory.getAuthenticationManager();
	}


	public static Map loadConfiguration(String configurationLocation, String appHomePath){

		Document doc = XMLUtils.read(configurationLocation);

		//authentication part
		Element authentication = doc.getRootElement().element(SecurityConstants.AUTHENTICATION);
		Map authenticationMap = new HashMap();
		authenticationMap.put(CoreConstants.DEBUG, authentication.element(CoreConstants.DEBUG).getTextTrim());
		authenticationMap.put(CoreConstants.INCLUDE_OLD_CONFIG, authentication.element(CoreConstants.INCLUDE_OLD_CONFIG).getTextTrim());
		authenticationMap.put(CoreConstants.INCLUDE_CONFIG_FROM_JAVA_PARAM, authentication.element(CoreConstants.INCLUDE_CONFIG_FROM_JAVA_PARAM).getTextTrim());
		authenticationMap.put(SecurityConstants.INCLUDE_POLICY_FROM_JAVA_PARAM, authentication.element(SecurityConstants.INCLUDE_POLICY_FROM_JAVA_PARAM).getTextTrim());
		if(authentication.element(SecurityConstants.DIGEST_ALGORITHM)!=null){
			authenticationMap.put(SecurityConstants.DIGEST_ALGORITHM,authentication.element(SecurityConstants.DIGEST_ALGORITHM).getTextTrim());
		}
		if(authentication.element(SecurityConstants.SALT)!=null){
			authenticationMap.put(SecurityConstants.SALT,authentication.element(SecurityConstants.SALT).getTextTrim());
		}
		//loginModules configuration
		List loginModuleElementsList = authentication.element(CoreConstants.LOGIN_MODULES).elements(SecurityConstants.LOGIN_MODULE);
		List loginModules = new ArrayList();
		Iterator itLoginModuleElementsList = loginModuleElementsList.iterator();
		while(itLoginModuleElementsList.hasNext()){
			Element loginModule = (Element)itLoginModuleElementsList.next();

			Map loginModuleMap = new HashMap();
			loginModuleMap.put(CoreConstants.NAME,loginModule.element(CoreConstants.NAME).getTextTrim());
			loginModuleMap.put(CoreConstants.FLAG,loginModule.element(CoreConstants.FLAG).getTextTrim());
			Element loginModuleOpts = loginModule.element(CoreConstants.LOGIN_MODULE_OPTIONS);
			if(loginModuleOpts!=null){
				List loginModuleOptsList = loginModuleOpts.elements(SecurityConstants.OPTION);
				Iterator itLoginModuleOpts = loginModuleOptsList.iterator();
				Map loginModulesOptions = new HashMap();
				while (itLoginModuleOpts.hasNext()){
					Element option = (Element)itLoginModuleOpts.next();
					String name = option.element(CoreConstants.NAME).getTextTrim();
					String value = option.element(SecurityConstants.VALUE_MARKUP).getTextTrim();
					if(SecurityConstants.AUTHENTICATION_XML_FILE_LOCATION.equals(name)||SecurityConstants.AUTHENTICATION_DATABASE_FILE_LOCATION.equals(name)){
						value=appHomePath+value;
					}
					loginModulesOptions.put(name,value);
				}
				loginModuleMap.put(CoreConstants.LOGIN_MODULE_OPTIONS, loginModulesOptions);
			}else{
				//there are no options for the loginmodule
				loginModuleMap.put(CoreConstants.LOGIN_MODULE_OPTIONS,new HashMap());
			}
			loginModules.add(loginModuleMap);
		}

		authenticationMap.put(CoreConstants.LOGIN_MODULES, loginModules);

		//authenticationManager configuration
		authenticationMap.put(SecurityConstants.AUTHENTICATION_MANAGER, authentication.element(SecurityConstants.AUTHENTICATION_MANAGER).getTextTrim());

		Map authenticationManagerOptions = new HashMap();
		Element authentManagerOptsElement = authentication.element(SecurityConstants.AUTHENTICATION_MANAGER_OPTIONS);
		List authentManagerOptsList = authentManagerOptsElement.elements(SecurityConstants.OPTION);
		Iterator authentManagerOpts = authentManagerOptsList.iterator();
		while (authentManagerOpts.hasNext()){
			Element option = (Element)authentManagerOpts.next();
			String name = option.element(CoreConstants.NAME).getTextTrim();
			String value = option.element(SecurityConstants.VALUE_MARKUP).getTextTrim();
			if(SecurityConstants.AUTHENTICATION_XML_FILE_LOCATION.equals(name)||SecurityConstants.AUTHENTICATION_DATABASE_FILE_LOCATION.equals(name)){
     	    	value=appHomePath+value;
     	    }
			authenticationManagerOptions.put(name,value);
		}

		authenticationMap.put(SecurityConstants.AUTHENTICATION_MANAGER_OPTIONS, authenticationManagerOptions);

		return authenticationMap;
	}
}
