/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;



import java.util.Map;
import java.util.logging.Logger;

import net.sf.jguard.ext.authentication.AuthenticationException;



/**
 * build an AuthenticationManager implementation.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class AuthenticationManagerFactory {

    private static AuthenticationManager authenticationManager;
    private static Logger logger = Logger.getLogger(AuthenticationManagerFactory.class.getName());

    /**
     * Convenience method that returns the current AuthenticationManager.
     * implementation.
     *
     * @return authenticationManager or<code><strong>null</strong></code> when
     * no authenticationManager is created.
     */
    public static AuthenticationManager getAuthenticationManager() {
        return authenticationManager;
    }

    /**
     * initialize the DAO implementation.
     * @param authenticationManagerImpl
     * @param map
     * @return <code>true</code> if AuthenticationManager is created successfully, <code>false</code> otherwise
     * @throws AuthenticationException
     */
    public static boolean createAuthenticationManager(String authenticationManagerImpl,Map map) throws AuthenticationException{
    	if(authenticationManager!=null){
    		return false;
    	}
           logger.info("authenticationManagerImpl="+authenticationManagerImpl);
           if(authenticationManagerImpl==null || authenticationManagerImpl.equals("")){
        	   throw new AuthenticationException(" authenticationManager implementation is null or empty "+authenticationManagerImpl);
           }
           try {

        	   authenticationManager = (AuthenticationManager)Class.forName(authenticationManagerImpl).newInstance();
        	   authenticationManager.init(map);

            } catch (InstantiationException e) {
                System.out.println("InstantiationException DAOImpl problem"+e.getMessage());
                throw new AuthenticationException(" InstantiationException DAOImpl "+e.getMessage(),e);
            } catch (IllegalAccessException e) {
                System.out.println("IllegalAccessException DAOImpl problem"+e.getMessage());
                throw new AuthenticationException(" IllegalAccessException DAOImpl "+e.getMessage(),e);
            } catch (ClassNotFoundException e) {
                System.out.println("ClassNotFoundException DAOImpl problem"+e.getMessage());
                throw new AuthenticationException(" ClassNotFoundException DAOImpl "+e.getMessage(),e);
            }
            return true;
        }
}
