/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name: v090beta3 $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;

import java.security.Principal;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.security.auth.Subject;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.authentication.AuthenticationException;
import net.sf.jguard.ext.authentication.SubjectTemplate;
import net.sf.jguard.ext.util.JNDIUtils;

/**
 *AuthenticationManager implementation providing a JNDI backend.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class JNDIAuthenticationManager extends AbstractAuthenticationManager {

	private static final Logger logger = Logger.getLogger(JNDIAuthenticationManager.class.getName());
    private DirContext initDirCtx = null;
	String applicationPath = null;
	String relativeRolePathPattern = null;
	String relativeRolesPath = "";
	private String ppalNameAttribute="cn";
	String searchRoot =null;
	String roleClass="organizationalRole";
	String userClass="inetOrgPerson";
	private int ppalAppNameLevel = 2;
	private Hashtable initParams = null;

	protected void persistUser(Subject user) throws AuthenticationException {
		logger.info(" this JNDIAUthenticationManager version is read-only ");

	}

	protected void persistPrincipal(Principal principal)
			throws AuthenticationException {
		if (principal instanceof RolePrincipal) {
			DirContext newContext = null;
			DirContext rolesContext =null;
			try {
				RolePrincipal jgPrincipal = (RolePrincipal) principal;

				rolesContext = (DirContext) initDirCtx.lookup(relativeRolesPath+applicationPath);
				Attributes atts = new BasicAttributes();
				atts.put("cn",jgPrincipal.getName());
				Attribute objectClassAtt = new BasicAttribute("objectClass",true);
				objectClassAtt.add("organizationalRole");
				objectClassAtt.add("top");
				atts.put(objectClassAtt);
				newContext = rolesContext.createSubcontext("cn="+jgPrincipal.getName(),atts);

			} catch (NamingException e) {
				throw new AuthenticationException(e.getMessage());
			}finally{
				try {
					rolesContext.close();
					if(newContext!= null){
						newContext.close();
	}
				} catch (NamingException e) {
					throw new AuthenticationException(e.getMessage());
				}
			}
		}else{
			logger.warning(" only jGuardPrincipal are supported in this version ");
			return;
		}

	}

	private String getPrincipalDN(RolePrincipal principal){
		StringBuffer resolvedDN = new StringBuffer();
		String relativeRolePath = MessageFormat.format(relativeRolePathPattern,new String[]{principal.getName()});
		resolvedDN.append(relativeRolePath).append(",");
		resolvedDN.append(relativeRolesPath);
		resolvedDN.append(applicationPath);
		return resolvedDN.toString();
	}



	public void persistSubjectTemplate(SubjectTemplate template) {
		logger.info(" this JNDIAUthenticationManager version is read-only ");

	}

	public void init(Map map) {

		Set entries =map.entrySet();
		Iterator itEntries = entries.iterator();
		Hashtable contextParams  = new Hashtable();
		//convert opts into an HashTable
		String applicationPathPattern = null;
		while(itEntries.hasNext()){
			Map.Entry entry = (Map.Entry)itEntries.next();
			if(entry.getValue()!=null){
				contextParams.put(entry.getKey(),entry.getValue());
				if(((String)entry.getKey()).equals("applicationPathPattern")){
					applicationPathPattern = (String)entry.getValue();
				}else if(((String)entry.getKey()).equals("relativeRolePathPattern")){
					relativeRolePathPattern=(String)entry.getValue();
				}else if(((String)entry.getKey()).equals("relativeRolesPath")){
					relativeRolesPath=(String)entry.getValue()+",";
				}else if(((String)entry.getKey()).equals("searchRoot")){
					searchRoot=(String)entry.getValue();
				}else if(((String)entry.getKey()).equals("ppalAppNameLevel")){
					ppalAppNameLevel=Integer.parseInt((String)entry.getValue());
				}else if(((String)entry.getKey()).equals(CoreConstants.APPLICATION_NAME)){
					applicationName= (String)entry.getValue();;
				}
			}
		}
		initParams = contextParams;
		applicationPath= MessageFormat.format(applicationPathPattern,new String[]{this.applicationName});

        try {
			initDirCtx = new InitialDirContext(contextParams);
		} catch (NamingException e) {
			e.printStackTrace();
			logger.severe(e.getMessage());
		}
	}

	public void updateUserImpl(JGuardCredential cred,Subject user) throws AuthenticationException {
		logger.info(" this JNDIAUthenticationManager version is read-only ");

	}

	public void deleteUser(Subject user) throws AuthenticationException {
		logger.info(" this JNDIAUthenticationManager version is read-only ");

	}

	public boolean isEmpty() {
		if(this.getAllPrincipalsSet().size()==0){
			logger.info(" there is no principals registered in the JNDI datasource ");
			return true;
		}
		if(this.getUsers().size()==0){
			logger.info(" there is no users registered in the JNDI datasource ");
			return true;
		}

		return false;
	}

	public SubjectTemplate getSubjectTemplate(String name) {
		logger.info(" feature not yet implemented ");
		return null;
	}

	/**
	 * return all the principals found in the Directory from all the applications.
	 * @return RolePrincipal set
	 */
	public Set getAllPrincipalsSet() {
		Set allPrincipals = new HashSet();
		SearchControls controls = new SearchControls();
		controls.setSearchScope(SearchControls.SUBTREE_SCOPE);
		String filter = "(objectClass="+roleClass+")";
		filter = JNDIUtils.escapeSearchFilter(filter);
		try {
			NamingEnumeration results = initDirCtx.search(searchRoot,filter, controls);
			while(results.hasMore()){
				SearchResult result  =(SearchResult)results.next();
				logger.finest("name in namespace"+result.getNameInNamespace());
				String applicationName = JNDIUtils.getValueInNameSpace(result,0);
				logger.finest("applicationName="+applicationName);
				Attributes attributes = result.getAttributes();
				 NamingEnumeration results2 = attributes.getAll();
				RolePrincipal principal = new RolePrincipal();
				principal.setApplicationName(applicationName);
				while(results2.hasMoreElements()){
					Attribute att = (Attribute)results2.next();
					String attributeID= att.getID();
					String attributeValue= (String)att.get();
					logger.finest("principal attributeID="+attributeID);
					logger.finest("principal attributeValue="+attributeValue);

					if(attributeID.equals(ppalNameAttribute)){
						principal.setName(attributeValue);
						logger.finest("principal name="+attributeValue);
					}
				}
				allPrincipals.add(principal);
			}
		} catch (NamingException e) {
			logger.severe(e.getMessage());
		}
		return allPrincipals;
	}


	public Set findUsers(Collection credentials) {
		Set users = new HashSet();
		SearchControls controls = new SearchControls();
		controls.setSearchScope(SearchControls.SUBTREE_SCOPE);
		String filterExpression = buildUserFilterExpression(credentials);
		logger.finest("filterExpression="+filterExpression);
		try {
			NamingEnumeration results = initDirCtx.search(searchRoot,filterExpression, controls);
			while(results.hasMore()){
				Subject subject = new Subject();
				SearchResult result  =(SearchResult)results.next();
				String userDn = result.getNameInNamespace();
				logger.finest("user dn found="+userDn);
				String filterExpression2 = "(objectClass="+roleClass+")";
				filterExpression2 = JNDIUtils.escapeSearchFilter(filterExpression2);
				NamingEnumeration results2 = initDirCtx.search(userDn,filterExpression2,controls);
				Set principals = subject.getPrincipals();
				principals.addAll(JNDIUtils.grabPrincipalsFromUser(results2,ppalNameAttribute,ppalAppNameLevel ));
				Set privateCredentials = subject.getPrivateCredentials();
				privateCredentials.addAll(JNDIUtils.grabCredentialsFromJNDIAttributes(result.getAttributes().getAll()));
				users.add(subject);
	}
		} catch (NamingException e) {
			logger.severe(e.getMessage());
		}
		return users;
	}

	private String buildUserFilterExpression(Collection credentials){
		StringBuffer filterExpression =new StringBuffer("(objectClass="+userClass+")");
		Iterator itCredentials = credentials.iterator();
		int i = 0;
		while(itCredentials.hasNext()){
			if(i==0){
				filterExpression.insert(0,"(&");
			}
			JGuardCredential cred = (JGuardCredential)itCredentials.next();
			filterExpression.append(" (");
			filterExpression.append(cred.getId());
			filterExpression.append("=");
			filterExpression.append(cred.getValue());
			filterExpression.append(")");
			i++;
		}
		if(i>0){
			filterExpression.append(")");
		}
		return filterExpression.toString();
	}

	public Set getUsers() {
		return findUsers(new ArrayList());
	}

	/**
     * change principal's name.
     * @param oldPrincipalName
     * @param principal
     * @throws AuthenticationException
     */
	public void updatePrincipal(String oldPrincipalName, Principal principal) throws AuthenticationException{
		if(!oldPrincipalName.equals(principal.getName())){
			logger.finest("principal name="+principal.getName());
			RolePrincipal oldPrincipal = new RolePrincipal();
			oldPrincipal.setName(oldPrincipalName);
			oldPrincipal.setApplicationName(this.applicationName);
			DirContext principalDirContext =null;
			try {
				principalDirContext = getPrincipalDirContext(oldPrincipal);
				System.out.println("oldName="+principalDirContext.getNameInNamespace());
				System.out.println("newName="+getPrincipalDN((RolePrincipal)principal));
				principalDirContext.rename(principalDirContext.getNameInNamespace(),getPrincipalDN((RolePrincipal)principal));
			} catch (NamingException e) {
				throw new AuthenticationException(e);
			}finally{
				try {
					if(principalDirContext!= null){
					principalDirContext.close();
					}
				} catch (NamingException e) {
					logger.severe(e.getMessage());
					throw new AuthenticationException(e);
				}
			}
		}else{
			logger.info(" principal name hasn't changed ");
		}

	}


		private DirContext getPrincipalDirContext(RolePrincipal oldPrincipal) throws NamingException {
		DirContext oldDirContext = null;
		oldDirContext = (DirContext) initDirCtx.lookup(getPrincipalDN(oldPrincipal));
		return oldDirContext;
	}

	public void deletePrincipal(Principal principal) {
		logger.info(" this JNDIAUthenticationManager version is read-only ");

	}


}
