/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authorization;

import java.security.Policy;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sf.ehcache.CacheException;
import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authorization.permissions.PermissionUtils;
import net.sf.jguard.core.authorization.policy.MultipleAppPolicy;
import net.sf.jguard.core.authorization.policy.PolicyHelper;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authorization.manager.AuthorizationManager;
import net.sf.jguard.ext.util.XMLUtils;

import org.dom4j.Document;
import org.dom4j.Element;

public class AuthorizationHelper {

	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger
	.getLogger(AuthorizationHelper.class.getName());


	public static AuthorizationManager initAuthorization(Map authorizationOptions,String applicationName){
		PolicyHelper.installPolicy();

		if ( !("true".equals(authorizationOptions.get(SecurityConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING)))){
			PermissionUtils.setCachesEnabled(false);
		}else{
			// by default, permission resolution caching is activated
			try {
				PermissionUtils.createCaches();
				PermissionUtils.setCachesEnabled(true);
			} catch (CacheException e) {
				logger.warning("Failed to activate permission resolution caching : " + e.getMessage());
				PermissionUtils.setCachesEnabled(false);
			}
		}

		AuthorizationManager am = null;
		//init authorization part
		try {
			// Create new authorization manager based on settings
			authorizationOptions.put(CoreConstants.APPLICATION_NAME,applicationName);
			if (logger.isLoggable(Level.FINEST)) {
				logger.finest(" authorizationManagerOptions="+ authorizationOptions);
			}
			AuthorizationManagerFactory.createAuthorizationManager(authorizationOptions);
			am = AuthorizationManagerFactory.getAuthorizationManager();

		} catch (AuthorizationException e) {
			logger.severe("AuthorizationException : "+e.getMessage());
			// TODO throw exception
		}

		//Register the new authorization manager with jguard policy provider
		MultipleAppPolicy jGuardPolicy = (MultipleAppPolicy)Policy.getPolicy();
		jGuardPolicy.registerPermissionProvider(Thread.currentThread().getContextClassLoader(),am);

		return am;
	}

	/**
	 * loads the Policy configuration file and returns settings.
	 * @param configurationLocation
	 * @param appHomePath
	 * @return map containing authorization settings
	 */
	public static Map loadConfiguration(String configurationLocation,String appHomePath){
		Map authorizationMap = new HashMap();
		Document doc = XMLUtils.read(configurationLocation);

		Element authorization = doc.getRootElement().element(SecurityConstants.AUTHORIZATION);

		Element permissionResolutionCaching = authorization.element(SecurityConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING);
		if (permissionResolutionCaching != null){
			authorizationMap.put(SecurityConstants.AUTHORIZATION_PERMISSION_RESOLUTION_CACHING, permissionResolutionCaching.getTextTrim());
		}
		authorizationMap.put(SecurityConstants.AUTHORIZATION_MANAGER,authorization.element(SecurityConstants.AUTHORIZATION_MANAGER).getTextTrim());

		List authorizationList = authorization.element(SecurityConstants.AUTHORIZATION_MANAGER_OPTIONS).elements(SecurityConstants.OPTION);
		Iterator itAuthorizationOptions = authorizationList.iterator();
		while(itAuthorizationOptions.hasNext()){
			Element option = (Element)itAuthorizationOptions.next();
			String name = option.element(CoreConstants.NAME).getTextTrim();
			String value = option.element(SecurityConstants.VALUE_MARKUP).getTextTrim();
			 if(SecurityConstants.AUTHORIZATION_XML_FILE_LOCATION.equals(name)||SecurityConstants.AUTHORIZATION_DATABASE_FILE_LOCATION.equals(name)){
	    	    	value=appHomePath+value;
	    	    }
			authorizationMap.put(name,value);
		}
		return authorizationMap;
	}

}
